(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2009 Xavier Clerc.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://trolltech.com/products/qt/licenses/licensing/qpl>.
 *)

(** Compilation from macroinstructions to Java bytecode. *)

type error =
  | Method_too_long of string

exception Error of error


val compile_function : Macroinstr.fundecl -> BaristaLibrary.Method.t
(** Translates the passed (macroinstruction-level) function into its
   (Java-bytecode) equivalent. *)

val reset_constants : unit -> unit
(** Resets the constants. *)

val compile_class : BaristaLibrary.Method.t list -> string
(** Builds a class from environment and the passed method list.
   Returns the compiled class as an array of bytes. *)

val compile_constants : unit -> string
(** Compiles the constants, returning them as an array of bytes. *)

val report_error : Format.formatter -> error -> unit
(** Pretty-prints an error. *)
