(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2009 Xavier Clerc.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://trolltech.com/products/qt/licenses/licensing/qpl>.
 *)

(** Some constants and utility functions for bytecode compilation. *)


(** {6 Name functions} *)

val normalize_function_name : string -> BaristaLibrary.Utils.utf8
(** Normalizes the passed function name such that it is a correct Java method
   name. *)

val make_class : string -> BaristaLibrary.Name.for_class
(** Constructs a class name from the passed string. *)

val make_field : string -> BaristaLibrary.Name.for_field
(** Constructs a field name from the passed string. *)

val make_method : string -> BaristaLibrary.Name.for_method
(** Constructs a method name from the passed string. *)


(** {6 Class constants} *)

val class_AbstractNativeApplet : BaristaLibrary.Name.for_class
val class_AbstractNativeRunner : BaristaLibrary.Name.for_class
val class_AbstractNativeServlet : BaristaLibrary.Name.for_class
val class_Application : BaristaLibrary.Name.for_class
val class_Block : BaristaLibrary.Name.for_class
val class_Class : BaristaLibrary.Name.for_class
val class_Closure : BaristaLibrary.Name.for_class
val class_CodeRunner : BaristaLibrary.Name.for_class
val class_Fail : BaristaLibrary.Name.for_class
val class_FailException : BaristaLibrary.Name.for_class
val class_FatalException : BaristaLibrary.Name.for_class
val class_InputStream : BaristaLibrary.Name.for_class
val class_Map : BaristaLibrary.Name.for_class
val class_NativeParameters : BaristaLibrary.Name.for_class
val class_Object : BaristaLibrary.Name.for_class
val class_PrintStream : BaristaLibrary.Name.for_class
val class_String : BaristaLibrary.Name.for_class
val class_Value : BaristaLibrary.Name.for_class
val class_Values : BaristaLibrary.Name.for_class


(** {6 Field constants} *)

val field_DOUBLE_ARRAY_TAG : BaristaLibrary.Instruction.t list
val field_DOUBLE_SIZE : BaristaLibrary.Instruction.t list
val field_ERR : BaristaLibrary.Instruction.t list
val field_FALSE : BaristaLibrary.Instruction.t list
val field_IN : BaristaLibrary.Instruction.t list
val field_MINUS_ONE : BaristaLibrary.Instruction.t list
val field_MINUS_TWO : BaristaLibrary.Instruction.t list
val field_ONE : BaristaLibrary.Instruction.t list
val field_OUT : BaristaLibrary.Instruction.t list
val field_TRUE : BaristaLibrary.Instruction.t list
val field_TWO : BaristaLibrary.Instruction.t list
val field_UNIT : BaristaLibrary.Instruction.t list
val field_ZERO : BaristaLibrary.Instruction.t list
val field_applet_runner : BaristaLibrary.Instruction.t list
val field_servlet_runner : BaristaLibrary.Instruction.t list

val field_size : int
val field_stack : int


(** {6 Method constants} *)

val cstr_AbstractNativeApplet : BaristaLibrary.Instruction.t list
val cstr_AbstractNativeRunner : BaristaLibrary.Instruction.t list
val cstr_AbstractNativeServlet : BaristaLibrary.Instruction.t list
val cstr_copy_AbstractNativeRunner : BaristaLibrary.Instruction.t list
val meth_asBlock : BaristaLibrary.Instruction.t list
val meth_asCodeOffset : BaristaLibrary.Instruction.t list
val meth_asDouble : BaristaLibrary.Instruction.t list
val meth_asLong : BaristaLibrary.Instruction.t list
val meth_asValue : BaristaLibrary.Instruction.t list
val meth_caml_make_array : BaristaLibrary.Instruction.t list
val meth_checkSignals : BaristaLibrary.Instruction.t list
val meth_createApplication : BaristaLibrary.Instruction.t list
val meth_createBlock : BaristaLibrary.Instruction.t list
val meth_createClosure : BaristaLibrary.Instruction.t list
val meth_createClosureVars : BaristaLibrary.Instruction.t list
val meth_createDouble : BaristaLibrary.Instruction.t list
val meth_createDoubles : BaristaLibrary.Instruction.t list
val meth_createDoubleArray : BaristaLibrary.Instruction.t list
val meth_createFromBlock : BaristaLibrary.Instruction.t list
val meth_createFromLong : BaristaLibrary.Instruction.t list
val meth_createFromRawValue : BaristaLibrary.Instruction.t list
val meth_createGlobal : BaristaLibrary.Instruction.t list
val meth_createInfix : BaristaLibrary.Instruction.t list
val meth_createValues : BaristaLibrary.Instruction.t list
val meth_equalValues : BaristaLibrary.Instruction.t list
val meth_execute : BaristaLibrary.Instruction.t list
val meth_executeWithBindings : BaristaLibrary.Instruction.t list
val meth_fromStream : BaristaLibrary.Instruction.t list
val meth_get : BaristaLibrary.Instruction.t list
val meth_getAtom : BaristaLibrary.Instruction.t list
val meth_getConstant : BaristaLibrary.Instruction.t list
val meth_getDouble : BaristaLibrary.Instruction.t list
val meth_getGlobal : BaristaLibrary.Instruction.t list
val meth_getHeader : BaristaLibrary.Instruction.t list
val meth_getMagicNumber : BaristaLibrary.Instruction.t list
val meth_getRawValue : BaristaLibrary.Instruction.t list
val meth_getResourceAsStream : BaristaLibrary.Instruction.t list
val meth_getResult : BaristaLibrary.Instruction.t list
val meth_getTag : BaristaLibrary.Instruction.t list
val meth_getUnsignedByte : BaristaLibrary.Instruction.t list
val meth_getWoSize : BaristaLibrary.Instruction.t list
val meth_greaterEqualValue : BaristaLibrary.Instruction.t list
val meth_greaterThanValue : BaristaLibrary.Instruction.t list
val meth_isBlock : BaristaLibrary.Instruction.t list
val meth_isLong : BaristaLibrary.Instruction.t list
val meth_isOutValue : BaristaLibrary.Instruction.t list
val meth_loadConstant : BaristaLibrary.Instruction.t list
val meth_lowerEqualValue : BaristaLibrary.Instruction.t list
val meth_lowerThanValue : BaristaLibrary.Instruction.t list
val meth_notEqualValues : BaristaLibrary.Instruction.t list
val meth_offset : BaristaLibrary.Instruction.t list
val meth_raise : BaristaLibrary.Instruction.t list
val meth_raiseZeroDivide : BaristaLibrary.Instruction.t list
val meth_registerClosure : BaristaLibrary.Instruction.t list
val meth_registerClosure1 : BaristaLibrary.Instruction.t list
val meth_registerClosureN : BaristaLibrary.Instruction.t list
val meth_registerInfix1 : BaristaLibrary.Instruction.t list
val meth_registerInfixN : BaristaLibrary.Instruction.t list
val meth_registerPredefinedException : BaristaLibrary.Instruction.t list
val meth_returnApplication : BaristaLibrary.Instruction.t list
val meth_returnClosure : BaristaLibrary.Instruction.t list
val meth_returnDoubles : BaristaLibrary.Instruction.t list
val meth_returnValues : BaristaLibrary.Instruction.t list
val meth_set : BaristaLibrary.Instruction.t list
val meth_setCode : BaristaLibrary.Instruction.t list
val meth_setCustom : BaristaLibrary.Instruction.t list
val meth_setDouble : BaristaLibrary.Instruction.t list
val meth_setGlobal : BaristaLibrary.Instruction.t list
val meth_setParent : BaristaLibrary.Instruction.t list
val meth_setUnsignedByte : BaristaLibrary.Instruction.t list
val meth_signedToUnsigned : BaristaLibrary.Instruction.t list
val meth_universalLong : BaristaLibrary.Instruction.t list
val meth_universalUnsignedLong : BaristaLibrary.Instruction.t list

val meth_size : int
val meth_stack : BaristaLibrary.Instruction.t list -> int


(** {6 Utility functions} *)

val push_int : int -> BaristaLibrary.Instruction.t list * int * int
(** Returns the instruction list needed to push the passed integer.
   Also returns instruction list size and stack expanse. *)

val push_boxed_int : int -> BaristaLibrary.Instruction.t list * int * int
(** Returns the instruction list needed to push the passed integer as a value.
   Also returns instruction list size and stack expanse. *)

val aload : int -> BaristaLibrary.Instruction.t list * int * int
(** Returns the instruction list needed to load a reference from a given index.
   Also returns instruction list size and stack expanse. *)

val astore : int -> BaristaLibrary.Instruction.t list * int * int
(** Returns the instruction list needed to store a reference into a given index.
   Also returns instruction list size and stack expanse. *)

val iload : int -> BaristaLibrary.Instruction.t list * int * int
(** Returns the instruction list needed to load an int from a given index.
   Also returns instruction list size and stack expanse. *)

val istore : int -> BaristaLibrary.Instruction.t list * int * int
(** Returns the instruction list needed to store an int into a given index.
   Also returns instruction list size and stack expanse. *)

val iinc : int -> int -> BaristaLibrary.Instruction.t list * int * int
(** [iinx idx incr] returns the instruction list needed to increment the int at
   index [idx] by [incr].
   Also returns instruction list size and stack expanse. *)
