(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2009 Xavier Clerc.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://trolltech.com/products/qt/licenses/licensing/qpl>.
 *)

(** Definition of Cadmium primitives. *)

type error =
  | Undefined_class of string * (string option)
  | Undefined_primitive of string

exception Error of error
(** Exception raised when a primitive or class cannot be found.*)

val init : unit -> unit
(** Initializes the module. Prints the primitive table if the "-print-primitives"
    command-line switch has been passed to the executable.
    The primitive table does not contain the builtin primitives unless the
    "-nobuiltin" command-line switch has been passed to the executable.
    This function is automatically called at the first call to [get_class] if
    it has not already been called. *)

val get_class : string -> string
(** [get_class prim] returns the name of the Cadmium class containing the
   primitive named [prim]. Raises [Error] if such a class does not exist or
   if a referenced class cannot be found in the classpath. *)

val report_error : Format.formatter -> error -> unit
(** Pretty-prints an error. *)
