(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2009 Xavier Clerc.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://trolltech.com/products/qt/licenses/licensing/qpl>.
 *)

(** Java variant of the lambda code. *)

type function_label = string * string
(** First component is the class name and second one is the method name. *)

type jlambda =
  | Jvar of Ident.t
  | Jconst of Lambda.structured_constant
  | Jdirect_apply of function_label * jlambda list * Debuginfo.t
  | Jgeneric_apply of jlambda * jlambda list * Debuginfo.t
  | Jclosure of (function_label * int * Ident.t list * jlambda) list * jlambda list
  | Joffset of jlambda * int
  | Jlet of Ident.t * jlambda * jlambda
  | Jletrec of (Ident.t * jlambda) list  * jlambda
  | Jprim of Lambda.primitive * jlambda list * Debuginfo.t
  | Jswitch of jlambda * jlambda_switch
  | Jstaticfail of int * jlambda list
  | Jstaticcatch of int * Ident.t list * jlambda * jlambda
  | Jtrywith of jlambda * Ident.t * jlambda
  | Jifthenelse of jlambda * jlambda * jlambda
  | Jsequence of jlambda * jlambda
  | Jwhile of jlambda * jlambda
  | Jfor of Ident.t * jlambda * jlambda * Asttypes.direction_flag * jlambda
  | Jassign of Ident.t * jlambda
  | Jsend of Lambda.meth_kind * jlambda * jlambda * jlambda list * Debuginfo.t
and jlambda_switch =
    { js_index_consts : int array;
      js_actions_consts : jlambda array;
      js_index_blocks : int array;
      js_actions_blocks : jlambda array; }

type function_description =
    { fun_label : function_label;
      fun_arity : int;
      mutable fun_closed : bool;
      mutable fun_inline : (Ident.t list * jlambda) option; }

type value_approximation =
  | Value_closure of function_description * value_approximation
  | Value_tuple of value_approximation array
  | Value_unknown
  | Value_integer of int
  | Value_constptr of int
