(*
 * This file is part of Cafesterol.
 * Copyright (C) 2007-2009 Xavier Clerc.
 *
 * Cafesterol is free software; you can redistribute it and/or modify
 * it under the terms of the Q Public License as published by
 * Trolltech (with a change to choice of law).
 *
 * Cafesterol is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Q Public License for more details.
 *
 * You should have received a copy of the Q Public License
 * along with this program.  If not, see
 * <http://trolltech.com/products/qt/licenses/licensing/qpl>.
 *)

(** Macroinstructions of Java bytecode. *)

type boxed = Boxed | Unboxed
(** Whether the compilation of an expression should result in a boxed value
   or an unboxed int. *)

type call = Primitive | Native
(** Whether the called element is a primitive or another native code. *)

type expression =
  | Mconst of Lambda.structured_constant
  | Moffset of expression * int
  | Mdynamicoffset of expression * expression
  | Mcreateglobal of string * int
  | Mclosure of int * (Jlambda.function_label * int) list * expression list
  | Mreadlocal of int
  | Mreadlocalint of int
  | Mwritelocal of int * expression
  | Mwritelocalint of int * expression
  | Mcall of call * Jlambda.function_label * expression list * Debuginfo.t
  | Mprim of Lambda.primitive * expression list * Debuginfo.t
  | Mapply of expression list * Debuginfo.t
  | Msequence of expression * expression
  | Mifthenelse of expression * expression * expression
  | Mwhile of expression * expression
  | Mfor of int * Asttypes.direction_flag * int * expression
  | Munit
  | Mpoptolocal of int
  | Mpop
  | Mnop
  | Mtrywith of expression * int * expression * int list
  | Mstaticfail of int * expression list
  | Mstaticcatch of int * expression * int * expression
  | Mswitch of expression * int array * expression array * expression
  | Mint of expression
  | Mtag of expression

type fundecl =
    { fun_name : string;
      fun_args : Ident.t list;
      fun_body : expression;
      fun_max_locals : int;
      fun_garbage_index : int; }
